<?php
/*
 * Make sure to disable the display of errors in production code!
 */
//ini_set('display_errors', 1);
//ini_set('display_startup_errors', 1);
//error_reporting(E_ALL);

require_once __DIR__ . "/../vendor/autoload.php";



class FrmMolliePaymentsController {

    public static $min_version = '1.07.05';
    public static $db_version = 0;
    public static $db_opt_name = 'frm_pay_db_version';

    public static function load_hooks() {
        add_action('plugins_loaded', 'FrmMolliePaymentsController::load_lang');
        register_activation_hook(dirname(dirname(__FILE__)) . '/formidable-mollie.php', 'FrmMolliePaymentsController::install');

        if (is_admin()) {
            add_action('admin_menu', 'FrmMolliePaymentsController::menu', 26);
            add_filter('frm_nav_array', 'FrmMolliePaymentsController::frm_nav', 30);
            add_filter('plugin_action_links_formidable-mollie/formidable-mollie.php', 'FrmMolliePaymentsController::settings_link', 10, 2);
            add_action('after_plugin_row_formidable-mollie/formidable-mollie.php', 'FrmMolliePaymentsController::min_version_notice');
            add_action('admin_notices', 'FrmMolliePaymentsController::get_started_headline');
      //      add_action('admin_init', 'FrmMolliePaymentsController::include_updater', 1);
            add_action('wp_ajax_frmpay_install', 'FrmMolliePaymentsController::install');
            add_filter('set-screen-option', 'FrmMolliePaymentsController::save_per_page', 10, 3);
            add_action('wp_ajax_frm_payments_mollie_ipn', 'FrmMolliePaymentsController::mollie_ipn');
            add_action('wp_ajax_nopriv_frm_payments_mollie_ipn', 'FrmMolliePaymentsController::mollie_ipn');

            add_filter('frm_form_options_before_update', 'FrmMolliePaymentsController::update_options', 15, 2);
            add_action('frm_show_entry_sidebar', 'FrmMolliePaymentsController::sidebar_list');
        }

        add_action('frm_trigger_mollie_create_action', 'FrmMolliePaymentsController::create_payment_trigger', 10, 3);

        add_action('update_payment', 'FrmMolliePaymentsController::mollie_ipn', 10, 2);
    }

    function my_url_handler() {
        // Manually parse the URL request
        if (!empty($_SERVER['REQUEST_URI'])) {
            $urlvars = explode('/', $_SERVER['REQUEST_URI']);
        }

        // Check for querystring variables
        //  if((!empty($urlvars[1]))  && (!empty($urlvars[2])))
        if (!empty($urlvars[1])) {
            $_REQUEST[1] = $urlvars[0];

            if ($urlvars[1] == 'id') {
                // Set the $_REQUEST vars
                //  $_REQUEST['myid'] = $urlvars[2];
                // Redirect to custom plugin template for "item" view
                //  add_action('template_redirect', 'my_custom_item_template');
                do_action('update_payment');
            }

            // ... handle multiple types of URL by checking $urlvars[1] here
        }
    }

    public static function path() {
        return dirname(dirname(__FILE__));
    }

    public static function load_lang() {
        load_plugin_textdomain('frmmollie', false, 'formidable-mollie/languages/');
    }

    public static function menu() {
        $frm_settings = FrmMolliePaymentsHelper::get_settings();
        $menu = $frm_settings ? $frm_settings->menu : 'Formidable';
        add_submenu_page('formidable', $menu . ' | Mollie', 'Mollie', 'frm_view_entries', 'formidable-payments', 'FrmMolliePaymentsController::route');

        add_filter('manage_' . sanitize_title($menu) . '_page_formidable-payments_columns', 'FrmMolliePaymentsController::payment_columns');
    }

    public static function frm_nav($nav) {
        if (current_user_can('frm_view_entries')) {
            $nav['formidable-payments'] = 'Mollie';
        }

        return $nav;
    }

    public static function payment_columns($cols = array()) {
        add_screen_option('per_page', array('label' => __('Payments', 'frmmollie'), 'default' => 20, 'option' => 'formidable_page_formidable_payments_per_page'));

        return array(
            'cb' => '<input type="checkbox" />',
            'receipt_id' => __('Receipt ID', 'frmmollie'),
            'user_id' => __('User', 'frmmollie'),
            'item_id' => __('Entry', 'frmmollie'),
            'form_id' => __('Form', 'frmmollie'),
            'completed' => __('Completed', 'frmmollie'),
            'amount' => __('Amount', 'frmmollie'),
            'created_at' => __('Date', 'frmmollie'),
            'begin_date' => __('Begin Date', 'frmmollie'),
            'paysys' => __('Processor', 'frmmollie'),
        );
    }

    // Adds a settings link to the plugins page
    public static function settings_link($links, $file) {
        $settings = '<a href="' . admin_url('admin.php?page=formidable-settings') . '">' . __('Settings', 'frmmollie') . '</a>';
        array_unshift($links, $settings);

        return $links;
    }

    public static function min_version_notice() {
        $frm_version = is_callable('FrmAppHelper::plugin_version') ? FrmAppHelper::plugin_version() : 0;

        // check if Formidable meets minimum requirements
        if (version_compare($frm_version, self::$min_version, '>=')) {
            return;
        }

        $wp_list_table = _get_list_table('WP_Plugins_List_Table');
        echo '<tr class="plugin-update-tr active"><th colspan="' . $wp_list_table->get_column_count() . '" class="check-column plugin-update colspanchange"><div class="update-message">' .
        __('You are running an outdated version of Formidable. This plugin may not work correctly if you do not update Formidable.', 'frmmollie') .
        '</div></td></tr>';
    }

    public static function get_started_headline() {
        // Don't display this error as we're upgrading
        if (isset($_GET['action']) and $_GET['action'] == 'upgrade-plugin' and ! isset($_GET['activate']))
            return;

        $db_version = get_option(self::$db_opt_name);
        if ((int) $db_version < self::$db_version) {
            if (is_callable('FrmAppHelper::plugin_url')) {
                $url = FrmAppHelper::plugin_url();
            } else if (defined('FRM_URL')) {
                $url = FRM_URL;
            } else {
                return;
            }
            ?>
            <div class="error" id="frmpay_install_message" style="padding:7px;"><?php _e('Your Formidable Payments database needs to be updated.<br/>Please deactivate and reactivate the plugin to fix this or', 'frmmollie'); ?> <a id="frmpay_install_link" href="javascript:frmpay_install_now()"><?php _e('Update Now', 'frmmollie') ?></a></div>  
            <script type="text/javascript">
                function frmpay_install_now() {
                    jQuery('#frmpay_install_link').replaceWith('<img src="<?php echo $url ?>/images/wpspin_light.gif" alt="<?php _e('Loading&hellip;'); ?>" />');
                    jQuery.ajax({type: "POST", url: "<?php echo admin_url('admin-ajax.php') ?>", data: "action=frmpay_install",
                        success: function (msg) {
                            jQuery("#frmpay_install_message").fadeOut("slow");
                        }
                    });
                }
                ;
            </script>
            <?php
        }
    }

     public static function install($old_db_version = false) {
        require_once(self::path() . '/models/FrmMolliePaymentDb.php');
        $frm_payment_db = new FrmMolliePaymentDb();
        $frm_payment_db->upgrade($old_db_version);
    }

    private static function show($id) {
        if (!$id)
            die(__('Selecteer een betaling', 'frmmollie'));

        global $wpdb;
        $payment = $wpdb->get_row($wpdb->prepare("SELECT p.*, e.user_id FROM {$wpdb->prefix}frm_payments p LEFT JOIN {$wpdb->prefix}frm_items e ON (p.item_id = e.id) WHERE p.id=%d", $id));

        include(self::path() . '/views/payments/show.php');
    }

    private static function display_list($message = '', $errors = array()) {
        if (!class_exists('WP_List_Table'))
            die(__('Please upgrade to at least WordPress v3.1 to see your payments', 'frmmollie'));

        include_once( self::path() . '/helpers/FrmMolliePaymentsListHelper.php' );
        $title = __('Downloads', 'frmmollie');
        $wp_list_table = new FrmMolliePaymentsListHelper();

        $pagenum = $wp_list_table->get_pagenum();

        $wp_list_table->prepare_items();

        $total_pages = $wp_list_table->get_pagination_arg('total_pages');
        if ($pagenum > $total_pages && $total_pages > 0) {
            wp_redirect(add_query_arg('paged', $total_pages));
            exit;
        }

        include(self::path() . '/views/payments/list.php');
    }

    public static function save_per_page($save, $option, $value) {
        if ($option == 'formidable_page_formidable_payments_per_page')
            $save = (int) $value;
        return $save;
    }

    public static function create_payment_trigger($action, $entry, $form) {
        return self::maybe_redirect_for_payment($action->post_content, $entry, $form);
    }


    public static function maybe_redirect_for_payment($settings, $entry, $form) {
        if (is_admin() && !defined('DOING_AJAX')) {
            // don't send to Mollie if submitting from the back-end
            return;
        }

        $amount = self::get_amount($form, $settings);
        if (empty($amount)) {
            return;
        }

        // stop the emails if payment is required
        if (FrmMolliePaymentsHelper::stop_email_set($form, $settings)) {
            remove_action('frm_trigger_email_action', 'FrmNotification::trigger_email', 10, 3);
            add_filter('frm_to_email', 'FrmMolliePaymentsController::stop_the_email', 20, 4);
            add_filter('frm_send_new_user_notification', 'FrmMolliePaymentsController::stop_registration_email', 10, 3);
        }

        // save in global for use building redirect url
        global $frm_mollie_form_settings;
        $frm_mollie_form_settings = $settings;

        // trigger payment redirect after other functions have a chance to complete
        add_action('frm_after_create_entry', 'FrmMolliePaymentsController::redirect_for_payment', 50, 2);

        return true;
    }

  public static function redirect_for_payment($entry_id, $form_id) {
        global $frm_mollie_form_settings;

        $form = FrmForm::getOne($form_id);
        $invoer = FrmEntry::GetOne($entry_id);
        $amount = self::get_amount($form, $frm_mollie_form_settings);
        if (empty($amount)) {
            return;
        }

        $frm_mollie_settings = new FrmMolliePaymentSettings();

        // TeoS 3-8-2018: Make it possible for the form to be in test-mode so that new forms can be tested on a live website without putting the whole website in test.
        if ( $frm_mollie_form_settings['mollie_test_mode'] == 1 ){
            $apikey = $frm_mollie_settings->settings->mollie_test_key;
        } else {
            // follow the global settings for test/live.
            if ($frm_mollie_settings->settings->mollie_environment == 'live') {
                $apikey = $frm_mollie_settings->settings->mollie_live_key;
            } else {
                $apikey = $frm_mollie_settings->settings->mollie_test_key;
            }
        }

        if (!isset($apikey)) {
            exit;
        }

        $the_return_show_invoice_url = $frm_mollie_settings->settings->mollie_return_page;
        FrmMolliePaymentsHelper::log_message('padnaam: ' . dirname(__DIR__) . '/mollie/API/Autoloader.php');

      // TeoS MollieUpdate  $mollie = new Mollie_API_Client;
        $mollie = new \Mollie\Api\MollieApiClient();
        
        FrmMolliePaymentsHelper::log_message('Mollie client created');

        $mollie->setApiKey($apikey);

        $protocol = isset($_SERVER['HTTPS']) && strcasecmp('off', $_SERVER['HTTPS']) !== 0 ? "https" : "http";
        $hostname = $_SERVER['HTTP_HOST'];
        $path = dirname(isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : $_SERVER['PHP_SELF']);
        
        /*
         * Payment parameters:
         *   amount        Amount in EUROs. This example creates a € 10,- payment.
         *   description   Description of the payment.
         *   redirectUrl   Redirect location. The customer will be redirected there after the payment.
         *   webhookUrl    Webhook location, used to report when the payment changes state.
         *   metadata      Custom metadata that is stored with the payment.
         */
        try {
            $item_name = apply_filters('frm_content', $frm_mollie_form_settings['mollie_item_label'], $form, $entry_id);
            $item_name .= ' ' . apply_filters('frm_content', $frm_mollie_form_settings['mollie_item_name'], $form, $entry_id);
            if ($item_name == null)
            {
                $item_name = "{$hostname}" . ' betaling';
            }
            FrmMolliePaymentsHelper::log_message("Item name: " . $item_name);

            $the_return_show_invoice_url = $frm_mollie_settings->settings->mollie_return_page;

            // TeoS 3-8-2018 override the return URL that is globally set. But only if the form holds a override.
            if( !$frm_mollie_form_settings['mollie_return_page'] == '' ) {
                $the_return_show_invoice_url = $frm_mollie_form_settings['mollie_return_page'];
            }

            if ( $frm_mollie_form_settings['mollie_use_url'] == 1 && !$frm_mollie_form_settings['mollie_url_parameter'] == '' ) {

                if (strpos($the_return_show_invoice_url, '?') == 0) {
                    $the_return_show_invoice_url .= '?';
                }

                $the_return_show_invoice_url .= $frm_mollie_form_settings['mollie_url_parameter'] .'='. $invoer->item_key;
            }

            $redirectURL = "{$protocol}://{$hostname}/" . $the_return_show_invoice_url;  //something like http://www.teowin.nl/showinvoice?
            $webhookURL = "{$protocol}://{$hostname}/wp-admin/admin-ajax.php?action=frm_payments_mollie_ipn&id="; //id variable is attached by Mollie..
            $gateway = self::get_paymentkind($form, $frm_mollie_form_settings);

            $payment = $mollie->payments->create(array(
                "amount" => [
                            "currency" => "EUR",
                            "value" =>  $amount
                            ],
                "description" => $item_name,
                "redirectUrl" => $redirectURL,
                "webhookUrl" => $webhookURL,

              /*   "metadata" => array(
                    "entry_id" => $entry_id,
                    "form_id" => $form_id
                ), */
            ));

            FrmMolliePaymentsHelper::log_message('Payment created: ' . $payment->description); 
 
            // Payment should be succeeded here. If $payment-> id is empty at this point, something went wrong quite well.        
            global $wpdb;

            $invoice = $wpdb->insert($wpdb->prefix . 'frm_payments', array(
                'item_id' => $entry_id, 'amount' => (float) $amount,
                'paysys' => 'mollie', 'begin_date' => current_time('mysql', 1),
                'created_at' => current_time('mysql', 1),
                'receipt_id' => $payment->id, 'expire_date' => '0000-00-00',
                    )
            );
           FrmMolliePaymentsHelper::log_message('Invoice added to database: ' . $payment->description);  
        } catch (Mollie_API_Exception $e) {
            echo "API call failed: " . htmlspecialchars($e->getMessage()) . " on field " . htmlspecialchars($e->getField() . PHP_EOL . "  WebhookURL: " . $webhookURL . PHP_EOL ." ReturnURL: " .  $redirectURL );
            FrmMolliePaymentsHelper::log_message(__('ERROR: ' . "API call failed: " . htmlspecialchars($e->getMessage()) . " on field " . htmlspecialchars($e->getField()), 'frmmollie'));
            die();
        }

        // TeoS MollieUpdate    $mollie_url = $payment->getPaymentUrl();
        $mollie_url = $payment->getCheckoutURL();

        $mollie_url = apply_filters('formidable_mollie_url', $mollie_url, $entry_id, $form_id);


        if (is_callable('FrmAppHelper::plugin_version')) {
            $frm_version = FrmAppHelper::plugin_version();
        } else {
            global $frm_version; //global fallback
        }

        add_filter('frm_redirect_url', 'FrmMolliePaymentsController::redirect_url', 9, 3);
        $conf_args = array('mollie_url' => $mollie_url);

        if (defined('DOING_AJAX') && isset($form->options['ajax_submit']) && $form->options['ajax_submit'] && $_POST && isset($_POST['action']) && in_array($_POST['action'], array('frm_entries_create', 'frm_entries_update'))) {
            $conf_args['ajax'] = true;
        }

        FrmProEntriesController::confirmation('redirect', $form, $form->options, $entry_id, $conf_args);
    }

    public static function get_paymentkind($form, $settings = array())
    {
        global $wpdb;

        $fields = FrmField::getAll($wpdb->prepare('fi.form_id=%d', $form->id) . " and fi.type not in ('end_divider', 'divider', 'html', 'break', 'captcha', 'rte')", ' ORDER BY field_order');
        $workfield = null;

        if(isset($fields) and is_array($fields)) {
            foreach ($fields as $field) {
                if ($field->type == 'mollie-kind') {
                   $workfield = $field;
                }
            }
        }
        if (isset($workfield)) {
            $selectedpayment =  $_POST['item_meta'][$workfield->id];


            return $selectedpayment;
        }
    }

    public static function get_amount($form, $settings = array()) {
        if (empty($settings)) {
            // for reverse compatability
            $settings = $form->options;
        }
        $amount_field = isset($settings['mollie_amount_field']) ? $settings['mollie_amount_field'] : '';
        $amount = 0.00;
        if (!empty($amount_field) && isset($_POST['item_meta'][$amount_field])) {
            $amount = $_POST['item_meta'][$amount_field];
        } else if (isset($settings['mollie_amount'])) {
            $amount = $settings['mollie_amount'];
        }

        if (empty($amount)) {
            // no amount has been set
            return "0,00";
        }

        $frm_mollie_settings = new FrmMolliePaymentSettings();
        $currencies = FrmMolliePaymentsHelper::get_currencies($frm_mollie_settings->settings->mollie_currency);

        $total = 0.00;
        foreach ((array) $amount as $a) {
            $this_amount = trim($a);
            preg_match_all('/[0-9,]*\.?[0-9]+/', $this_amount, $matches);
            $this_amount = $matches ? end($matches[0]) : 0;
            $this_amount = round((float) $this_amount, $currencies['decimals']);
            $total += $amount;
            unset($a, $this_amount, $matches);
        }

        return (string) number_format((float)$total, 2, '.', ''); $total;
    }

    public static function redirect_url($url, $form, $args = array()) {
        if (isset($args['mollie_url'])) {
            //only change it if it came from this plugin
            $url = $args['mollie_url'];
        }

        return $url;
    }

    public static function stop_registration_email($send_it, $form, $entry_id) {
        if (!is_callable('FrmRegAppController::send_paid_user_notification')) {
            // don't stop the registration email unless the function exists to send it later
            return $send_it;
        }

        if (!isset($_POST['payment_completed']) || empty($_POST['payment_completed'])) {
            // stop the email if payment is not completed
            $send_it = false;
        }

        return $send_it;
    }

    public static function stop_the_email($emails, $values, $form_id, $args = array()) {
        if (isset($_POST['payment_completed']) && absint($_POST['payment_completed'])) {
            // always send the email if the payment was just completed
            return $emails;
        }

        $action = FrmAppHelper::get_post_param('action', '', 'sanitize_title');
        $frm_action = FrmAppHelper::get_post_param('frm_action', '', 'sanitize_title');
        if (isset($args['entry']) && $action == 'frm_entries_send_email') {
            // if resending, make sure the payment is complete first
            global $wpdb;
            $complete = FrmDb::get_var($wpdb->prefix . 'frm_payments', array('item_id' => $args['entry']->id, 'completed' => 1), 'completed');
        } else {
            // send the email when resending the email, and we don't know if the payment is complete
            $complete = (!isset($args['entry']) && ( $frm_action == 'send_email' || $action == 'frm_entries_send_email' ) );
        }

        //do not send if payment is not complete
        if (!$complete) {
            $emails = array();
        }

        return $emails;
    }

    //Trigger the email to send after a payment is completed:
    public static function send_email_now($vars, $payment, $entry) {
     // TeoS 03-08-2018
     //   if (!isset($vars['completed']) || !$vars['completed']) {
            //only send the email if payment is completed
     //       return;
     //   }
     // TeoS 03-08-2018

        $_POST['payment_completed'] = true; //to let the other function know to send the email

        if (is_callable('FrmFormActionsController::trigger_actions')) {
           
            FrmFormActionsController::trigger_actions('create', $entry->form_id, $entry->id, 'email');
        }

        if ($payment->status == "paid") {  // TeoS 03-08-2018: only if payment is paid.

            // trigger registration email

            if (is_callable('FrmRegAppController::send_paid_user_notification')) {
                FrmRegAppController::send_paid_user_notification($entry);
            }
        }
    }

    public static function mollie_ipn() {
        FrmMolliePaymentsHelper::log_message(__('Start van mollie_ipn.', 'frmmollie'));
      //  print_r($_GET);

        $frm_mollie_settings = new FrmMolliePaymentSettings();

        $loggin =  '';


        FrmMolliePaymentsHelper::log_message(__('POST variables.', 'frmmollie'));
        foreach ($_POST as $key => $value) {
            $loggin .= $key . " : " . $value . "<br />\r\n";
            FrmMolliePaymentsHelper::log_message(__('Variabeles in payment check: ' . $loggin, 'frmmollie'));
        }

        try {


            if ($frm_mollie_settings->settings->mollie_environment == 'live') {
                $apikey = $frm_mollie_settings->settings->mollie_live_key;
            } else {
                $apikey = $frm_mollie_settings->settings->mollie_test_key;
            }

            if (!isset($apikey)) {
                exit;
            }

       // TeoS MollieUpdate     $mollie = new Mollie_API_Client;
            $mollie = new \Mollie\Api\MollieApiClient();


            $mollie->setApiKey($apikey);


            FrmMolliePaymentsHelper::log_message(__('mollie object created.', 'frmmollie'));



            if (isset($_POST["id"])) {
                $paymentID = $_POST["id"];
            }

            if (!isset($paymentID)) {
                $paymentID = $_GET["id"];
        //            FrmMolliePaymentsHelper::log_message(__('GET ID = ' . $paymentID, 'frmmollie')); 
           }
           $payment = $mollie->payments->get($paymentID);

            $orderId = $payment->metadata->order_id;


            FrmMolliePaymentsHelper::log_message(__('mollie payment read.' . $payment->id, 'frmmollie'));


            $form_id = $payment->metadata->form_id;
            FrmMolliePaymentsHelper::log_message(__('mollie order id read.', 'frmmollie'));
        } catch (Mollie_API_Exception $e) {
            FrmMolliePaymentsHelper::log_message(__('Error: ' . htmlspecialchars($e->getMessage()), 'frmmollie'));
            echo "API call failed: " . htmlspecialchars($e->getMessage());
          
        }

        //   extract($_POST);
        //	$custom		= FrmAppHelper::get_post_param( 'custom', '', 'sanitize_text_field' );
        //	$business	= strtolower( FrmAppHelper::get_post_param( 'business', '', 'sanitize_text_field' ) );
        //	$receiver_email = strtolower( FrmAppHelper::get_post_param( 'receiver_email', '', 'sanitize_text_field' ) );
        //	$invoice	= FrmAppHelper::get_post_param( 'invoice', '', 'absint' );
        //	$txn_id		= FrmAppHelper::get_post_param( 'txn_id', '', 'sanitize_text_field' );


        $frm_mollie_settings->settings->mollie_email = strtolower($frm_mollie_settings->settings->mollie_email);

        //Check email address to make sure that this is not a spoof
        //	$mollie_email_match = ( $business == $frm_mollie_settings->settings->mollie_email );
        //	$receiver_email_match = ( $receiver_email == $frm_mollie_settings->settings->mollie_email );
        //	if ( empty( $custom ) || ( ! $mollie_email_match && ! $receiver_email_match ) ) {
        //		FrmPaymentsHelper::log_message( __( 'The receiving email address in the IPN does not match the settings.', 'frmmollie' ) );
        //		return;
        //	}
        //get entry associated with this payment
        //list( $entry_id, $hash ) = explode( '|', $custom );
        //$entry_id = absint( $entry_id );
        FrmMolliePaymentsHelper::log_message(__('Voor het ophalen van de betaling.' . $payment->id, 'frmmollie'));
        //mark as paid
     
        global $wpdb;

      $paymentinfo = $wpdb->get_results($wpdb->prepare("SELECT * FROM {$wpdb->prefix}frm_payments WHERE receipt_id='%s'", $paymentID));
      //      $wpdb->get_row("SELECT * FROM {$wpdb->prefix}frm_payments WHERE receipt_id='" . $paymentID . "'", $paymentinfo );
 //       FrmMolliePaymentsHelper::log_message('last query: ' . $wpdb->last_query);
    
        if (!$paymentinfo) {
            FrmMolliePaymentsHelper::log_message(__('The IPN does not match an existing payment.', 'frmmollie'));
            return;
        }
        else
        {
            FrmMolliePaymentsHelper::log_message('paymentinfo = OK');
            FrmMolliePaymentsHelper::log_message('paymentinfo->id ' . $paymentinfo[0]->id);
            FrmMolliePaymentsHelper::log_message('paymentinfo->item_id ' . $paymentinfo[0]->item_id);
        }
        
       

        /*
          TODO: process subscriptions
          switch(strtolower($txn_type)){
          case "subscr_payment" :
          case "subscr_signup" :
          case "subscr_cancel" :
          case "subscr_eot" : //expired
          case "subscr_failed" :
         */

        $pay_vars = (array) $paymentinfo[0];
        
        if (!$paymentinfo->receipt_id) {
            $pay_vars['receipt_id'] = $payment->id;
        }

      //  $pay_vars['meta_value'] = $payment->metadata;
        
     //   FrmAppHelper::get_post_param( 'payment_status', '', 'sanitize_text_field' );

      if ($payment->isPaid() && !$payment->hasRefunds() && !$payment->hasChargebacks()) {
        $pay_vars['completed'] = true;
      }
        $pay_vars['amount']    = $payment->amount->value;
        
        $next_payment_date = FrmAppHelper::get_post_param('next_payment_date', '', 'sanitize_text_field');
        if (isset($next_payment_date)) {
            $pay_vars['expire_date'] = date('Y-m-d H:i:s', strtotime($next_payment_date));
        }
        
        
         
        $entry_id = $pay_vars['item_id']; 
        $paymentrec_id = $pay_vars['id'];

        // TeoS 12-7-2018

        $entry = FrmEntry::getOne($entry_id);
     //   $entry->form_id;
        $form = FrmForm::getOne($entry->form_id);

        FrmEntry::get_meta($entry);

        $status_field = $form->options["mollie_betaalstatus_field"];
        // TeoS 12-7-2018

        FrmMolliePaymentsHelper::log_message('563 -> '. $entry_id);
        FrmMolliePaymentsHelper::log_message('564 -> '. $paymentrec_id);
        $entry = FrmEntry::getOne($entry_id);
        
        if (!$entry) {
            FrmMolliePaymentsHelper::log_message(__('The IPN does not match an existing entry.', 'frmpp'));
            return;
        }

        if ($wpdb->update($wpdb->prefix . 'frm_payments', $pay_vars, array('id' => $paymentrec_id)) === FALSE) 
        {
            FrmMolliePaymentsHelper::log_message(sprintf(__('Payment %d was complete, but failed to update.', 'frmmollie'), $paymentrec_id));
            FrmMolliePaymentsHelper::log_message('last query: ' . $wpdb->last_query);
            FrmMolliePaymentsHelper::log_message('last error: ' . $wpdb->last_error);
            return;
        } else {

            FrmMolliePaymentsHelper::log_message(__('Payment successfully updated.', 'frmmollie'));



            // TeoS 12-7-2018

            global $wpdb;



            $settings = $wpdb->get_results($wpdb->prepare("SELECT post_content FROM {$wpdb->prefix}posts WHERE post_excerpt='mollie' AND menu_order ='%s'", $entry->form_id));
            $fieldkey = 0;

            $instellingen = json_decode( $settings[0]->post_content );
            foreach ($instellingen as $setting => $fieldwaarde)
            {
                if ($setting === 'mollie_betaalstatus_field')
                {
                   $fieldkey = $fieldwaarde;
                }
            }

            if ($fieldkey > 0){  // only update if key field is set.
                $added = FrmEntryMeta::add_entry_meta( $entry_id, $fieldkey, null,  $payment->status );
                if ( ! $added ) {
                    FrmEntryMeta::update_entry_meta( $entry_id, 73, null, $payment->status );
                }
            }

            // TeoS 12-7-2018

            if (FrmMolliePaymentsHelper::stop_email_set($entry->form_id)) {
                self::send_email_now($pay_vars, $payment, $entry);
            }
        }
       
        do_action('frm_payment_mollie_ipn', compact('pay_vars', 'payment', 'entry'));
        die();
    }

    public static function hidden_payment_fields($form) {
        if (isset($form->options['mollie']) and $form->options['mollie']) {
            echo '<input type="hidden" name="frm_mollie_payment[item_name]" value="' . esc_attr($form->options['mollie_item_name']) . '"/>' . "\n";
        }
    }

    public static function setup_new_vars($values) {
        $defaults = FrmMolliePaymentsHelper::get_default_options();
        foreach ($defaults as $opt => $default) {
            $values[$opt] = FrmAppHelper::get_param($opt, $default);
            unset($default, $opt);
        }
        return $values;
    }

    public static function setup_edit_vars($values) {
        $defaults = FrmMolliePaymentsHelper::get_default_options();
        foreach ($defaults as $opt => $default) {
            if (!isset($values[$opt])) {
                $values[$opt] = ( $_POST && isset($_POST['options'][$opt]) ) ? $_POST['options'][$opt] : $default;
            }
            unset($default, $opt);
        }

        if ($values['mollie_item_name'] == '') {
            global $wpdb;
            $values['mollie_item_name'] = FrmDb::get_var($wpdb->prefix . 'frm_mollie_forms', array('id' => $values['id']), 'name');
        }

        return $values;
    }

    public static function update_options($options, $values) {
        $defaults = FrmMolliePaymentsHelper::get_default_options();

        foreach ($defaults as $opt => $default) {
            $options[$opt] = isset($values['options'][$opt]) ? $values['options'][$opt] : $default;
            unset($default, $opt);
        }

        return $options;
    }

    public static function sidebar_list($entry) {
        global $wpdb;

        $payments = $wpdb->get_results($wpdb->prepare("SELECT id,begin_date,amount,completed FROM {$wpdb->prefix}frm_payments WHERE item_id=%d ORDER BY created_at DESC", $entry->id));

        if (!$payments)
            return;

        $date_format = get_option('date_format');
        $currencies = FrmMolliePaymentsHelper::get_currencies();

        include(self::path() . '/views/payments/sidebar_list.php');
    }

    private static function new_payment() {
        self::get_new_vars();
    }

    private static function create() {
        $message = $error = '';

        require_once(self::path() . '/models/FrmMolliePayment.php');
        $frm_payment = new FrmMolliePayment();
        if ($id = $frm_payment->create($_POST)) {
            $message = __('Payment was Successfully Created', 'frmmollie');
            self::get_edit_vars($id, '', $message);
        } else {
            $error = __('There was a problem creating that payment', 'frmmollie');
            return self::get_new_vars($error);
        }
    }

    private static function edit() {
        $id = FrmAppHelper::get_param('id');
        return self::get_edit_vars($id);
    }

    private static function update() {
        require_once(self::path() . '/models/FrmMolliePayment.php');
        $frm_payment = new FrmMolliePayment();
        $id = FrmAppHelper::get_param('id');
        $message = $error = '';
        if ($frm_payment->update($id, $_POST))
            $message = __('Payment was Successfully Updated', 'frmmollie');
        else
            $error = __('There was a problem updating that payment', 'frmmollie');
        return self::get_edit_vars($id, $error, $message);
    }

    private static function destroy() {
        if (!current_user_can('administrator')) {
            $frm_settings = FrmMolliePaymentsHelper::get_settings();
            wp_die($frm_settings->admin_permission);
        }

        require_once(self::path() . '/models/FrmMolliePayment.php');
        $frm_payment = new FrmMolliePayment();
        $message = '';
        if ($frm_payment->destroy(FrmAppHelper::get_param('id')))
            $message = __('Payment was Successfully Deleted', 'frmmollie');

        self::display_list($message);
    }

    private static function bulk_actions($action) {
        $errors = array();
        $message = '';
        $bulkaction = str_replace('bulk_', '', $action);

        $items = FrmAppHelper::get_param('item-action', '');
        if (empty($items)) {
            $errors[] = __('No payments were selected', 'frmmollie');
        } else {
            if (!is_array($items))
                $items = explode(',', $items);

            if ($bulkaction == 'delete') {
                if (!current_user_can('frm_delete_entries')) {
                    $frm_settings = FrmMolliePaymentsHelper::get_settings();
                    $errors[] = $frm_settings->admin_permission;
                } else {
                    if (is_array($items)) {
                        require_once(self::path() . '/models/FrmMolliePayment.php');
                        $frm_payment = new FrmMolliePayment();
                        foreach ($items as $item_id) {
                            if ($frm_payment->destroy($item_id))
                                $message = __('Payments were Successfully Deleted', 'frmmollie');
                        }
                    }
                }
            }
        }
        self::display_list($message, $errors);
    }

    private static function get_new_vars($error = '') {
        global $wpdb;

        $defaults = array('completed' => 0, 'item_id' => '', 'receipt_id' => '', 'amount' => '', 'begin_date' => date('Y-m-d'), 'paysys' => 'manual');
        $payment = array();
        foreach ($defaults as $var => $default)
            $payment[$var] = FrmAppHelper::get_param($var, $default);

        $frm_mollie_settings = new FrmMolliePaymentSettings();
        $mollie_currency = FrmMolliePaymentsHelper::get_currencies($frm_mollie_settings->settings->mollie_currency);
        $users = FrmProFieldsHelper::get_user_options();

        require(self::path() . '/views/payments/new.php');
    }

    private static function get_edit_vars($id, $errors = '', $message = '') {
        if (!$id)
            die(__('Please select a payment to view', 'frmmollie'));

        if (!current_user_can('frm_edit_entries'))
            return self::show($id);

        global $wpdb;
        $payment = $wpdb->get_row($wpdb->prepare("SELECT p.*, e.user_id FROM {$wpdb->prefix}frm_payments p LEFT JOIN {$wpdb->prefix}frm_items e ON (p.item_id = e.id) WHERE p.id=%d", $id), ARRAY_A);

        $frm_mollie_settings = new FrmMolliePaymentSettings();
        $mollie_currency = FrmMolliePaymentsHelper::get_currencies($frm_mollie_settings->settings->mollie_currency);

        $users = FrmProFieldsHelper::get_user_options();

        if (isset($_POST) and isset($_POST['receipt_id'])) {
            foreach ($payment as $var => $val) {
                if ($var == 'id')
                    continue;
                $payment[$var] = FrmAppHelper::get_param($var, $val);
            }
        }

        require(self::path() . '/views/payments/edit.php');
    }

    public static function route() {
        $action = isset($_REQUEST['frm_action']) ? 'frm_action' : 'action';
        $action = FrmAppHelper::get_param($action);

        if ($action == 'show') {
            return self::show(FrmAppHelper::get_param('id', false));
        } else if ($action == 'new') {
            return self::new_payment();
        } else if ($action == 'create') {
            return self::create();
        } else if ($action == 'edit') {
            return self::edit();
        } else if ($action == 'update') {
            return self::update();
        } else if ($action == 'destroy') {
            return self::destroy();
        } else {
            $action = FrmAppHelper::get_param('action');
            if ($action == -1)
                $action = FrmAppHelper::get_param('action2');

            if (strpos($action, 'bulk_') === 0) {
                if (isset($_GET) and isset($_GET['action']))
                    $_SERVER['REQUEST_URI'] = str_replace('&action=' . $_GET['action'], '', $_SERVER['REQUEST_URI']);
                if (isset($_GET) and isset($_GET['action2']))
                    $_SERVER['REQUEST_URI'] = str_replace('&action=' . $_GET['action2'], '', $_SERVER['REQUEST_URI']);

                return self::bulk_actions($action);
            }else {
                return self::display_list();
            }
        }
    }

}
