<?php
/**
 * Created by Teowin 03-08-2018
 * User: Teo
 * Date: 8/3/2018
 * Purpose: Add a payment selection field to Formidable Forms.
 */

require_once __DIR__ . "/../vendor/autoload.php";

class FrmMolliePaymentsFormFieldsController{

    public static function load_hooks()
    {
        add_filter('frm_pro_available_fields', 'FrmMolliePaymentsFormFieldsController::add_pro_field');
        add_filter('frm_before_field_created', 'FrmMolliePaymentsFormFieldsController::set_mollie_field_defaults');

        add_filter('frm_update_field_options', 'FrmMolliePaymentsFormFieldsController::update_field_options', 10, 3);
        add_action('frm_display_added_fields', 'FrmMolliePaymentsFormFieldsController::show_the_admin_field');
        add_action('frm_field_options_form', 'FrmMolliePaymentsFormFieldsController::mollie_field_options_form', 10, 3);
        add_action('frm_form_fields', 'FrmMolliePaymentsFormFieldsController::show_my_front_field', 10, 3);

        add_filter('frm_setup_new_fields_vars', 'FrmMolliePaymentsFormFieldsController::frm_populate_posts', 20, 2);
        add_filter('frm_setup_edit_fields_vars', 'FrmMolliePaymentsFormFieldsController::frm_populate_posts', 20, 2); //use this function on edit too

        add_filter('frm_pre_create_entry', 'FrmMolliePaymentsFormFieldsController::frm_update_payment');
    }

    public static function add_pro_field($fields)
        {
            $fields['mollie-kind'] = __('Mollie Options'); // the key for the field and the label
            return $fields;
        }

    public static function show_the_admin_field($field){
        if ( $field['type'] != 'mollie-kind' ) {
            return;
        }

        $field_name = 'item_meta['. $field['id'] .']';
        ?>

        <div class="frm_html_field_placeholder">
            <div class="howto button-secondary frm_html_field">This is a placeholder for your payment options <br/>View your form to see it in action.</div>
        </div>

<?php






    }


    public static function set_mollie_field_defaults($field_data){
        if ( $field_data['type'] == 'mollie-kind' ) {
            $field_data['name'] = 'Betaalgateway';


            $defaults = '';

            foreach ($defaults as $k => $v) {
                $field_data['field_options'][$k] = $v;
            }

        }
        return $field_data;
    }


    public static function mollie_field_options_form($field, $display, $values){
        if ( $field['type'] != 'mollie-kind' ) {
            return;
        }

        $paymentoptions = $field['payment_opts'];

        // if the payment options array is empty, select them all.
        if (!isset($paymentoptions) | $paymentoptions == '') {
           unset($paymentoptions);
           $paymentoptions = array();
           foreach ($field['options'] as $test => $naam){
               $paymentoptions[htmlspecialchars($test)] = htmlspecialchars($naam);
           }
        }

?>

        <tr><td><label>Beschikbare betaalmethoden</label></td>
            <td>

              <select multiple name="field_options[payments_<?php echo $field['id'] ?>][]" id="field_options[size_<?php echo $field['id'] ?>][]">  <?php
              foreach($field['options'] as $test => $naam) {
                     if (in_array($naam, $paymentoptions)) {
                         echo '<option selected value="'.$test. '"> ' . $naam . '</option>';
                     }
                     else {
                         echo '<option value="' . $test . '"> ' . $naam . '</option>';
                     }
              }
             ?>
              </select>
            </td>
        </tr>
<?php

    }


    public static function update_field_options( $field_options, $field, $values )
    {
        if ($field->type != 'mollie-kind') {
           return $field_options;
        }
        $payments = '';
        $field_options['payment_opts'] = isset( $values['field_options'][ 'payments' . '_' . $field->id ] ) ? $values['field_options'][ 'payments' . '_' . $field->id ] : $payments;

        return $field_options;
    }


    function show_my_front_field( $field, $field_name, $atts ) {
        if ( $field['type'] != 'mollie-kind' ) {
            return;
        }
        $field['value'] = stripslashes_deep($field['value']);

        $paymentopts = $field['payment_opts'];

        global $paymentopts;

        ?>
        <select id="<?php echo esc_attr( $atts['html_id'] ) ?>" name="<?php echo esc_attr( $field_name ) ?>">
            <?php
            foreach($field['payment_opts'] as $test => $naam) {

                    echo '<option value="' . $test . '"> ' . $naam . '</option>';
                            }
            ?>
        </select>
        <?php
    }


    public static function frm_update_payment($values){

        global $wpdb;

        $fields = FrmField::getAll($wpdb->prepare('fi.form_id=%d', $values['form_id']) . " and fi.type not in ('end_divider', 'divider', 'html', 'break', 'captcha', 'rte')", ' ORDER BY field_order');
        $workfield = null;

        if(isset($fields) and is_array($fields)) {
            foreach ($fields as $field) {
                if ($field->type == 'mollie-kind') {
                    $workfield = $field;
                }
            }
        }

        if (isset($workfield)) {
            $selectedpayment =  $_POST['item_meta'][$workfield->id];

            $values['item_meta'][$workfield->id] = $workfield->field_options['payment_opts'][$selectedpayment];
            $_POST['item_meta'][$workfield->id] = $workfield->field_options['payment_opts'][$selectedpayment];
        }

    }

    public static function frm_populate_posts($values, $field){
        if($field->type == 'mollie-kind') { //replace 125 with the ID of the field to populate

            global $frm_mollie_form_settings;

            $frm_mollie_settings = new FrmMolliePaymentSettings();

            $apikey = $frm_mollie_settings->settings->mollie_test_key;

            /*
             * Get all the activated methods for this API key.
             */
            // TeoS MollieUpdate     $mollie = new Mollie_API_Client;
            $mollie = new \Mollie\Api\MollieApiClient();

            FrmMolliePaymentsHelper::log_message('Mollie client created');

            $mollie->setApiKey($apikey);
            $methods = $mollie->methods->all();
            $i = 0;
            unset($values['options']);
            $values['options'] = array();
            //  echo '<select multiple>';
            foreach ($methods as $method) {
                //  echo '<div style="line-height:40px; vertical-align:top">';
                //   echo '<img src="' . htmlspecialchars($method->image->normal) . '" srcset="' . htmlspecialchars($method->image->bigger) . ' "> ';
                // TeoS             echo '<img src="' . htmlspecialchars($method->image->size1x) . '" srcset="' . htmlspecialchars($method->image->size2x) . ' 2x"> ';

                //      htmlspecialchars($method->description) . ' (' . htmlspecialchars($method->id) . ')';
                //      echo '</div>';

                //   echo '<option style="background-image:url(' . htmlspecialchars($method->image->normal) . ') value="' . htmlspecialchars($method->id) . '">' . htmlspecialchars($method->description) . '</option>';
                $values['options'][htmlspecialchars($method->id)] = htmlspecialchars($method->description);

                $values['use_key'] = false; //this will set the field to save the post ID instead of post title
            };
            //   echo '</select>';

        }
            return $values;

    }

}